/* extension.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

import GObject from 'gi://GObject';
import St from 'gi://St';
const { Soup, GLib } = imports.gi;

import { Extension, gettext as _ } from 'resource:///org/gnome/shell/extensions/extension.js';
import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';

import * as Main from 'resource:///org/gnome/shell/ui/main.js';

const newMenuItem = (label) => {
    return new PopupMenu.PopupMenuItem(label);
}

const Indicator = GObject.registerClass(
    class Indicator extends PanelMenu.Button {
        _init() {
            super._init(0.0, _('MyPubIP'));

            this.add_child(new St.Icon({
                icon_name: 'face-smile-symbolic',
                style_class: 'system-status-icon',
            }));

            let itemIPv4 = new PopupMenu.PopupMenuItem(_('Get public IP address'));
            itemIPv4.connect('activate', () => {
                try {
                    const message = Soup.Message.new('GET', "https://api.ipify.org");
                    const session = new Soup.Session();
                    const response = session.send_and_read_async(
                        message,
                        GLib.PRIORITY_DEFAULT,
                        null,
                        (session, result) => {
                            if (message.get_status() === Soup.Status.OK) {
                                let bytes = session.send_and_read_finish(result);
                                let decoder = new TextDecoder('utf-8');
                                let response = decoder.decode(bytes.get_data());
                                console.log(`Response: ${response}`);
                                Main.notify(_('Whatʼs up? your ip is ' + response));
                            }
                        }

                    );

                } catch (e) {
                    console.log("Error: could not fetch public IP address: " + e);
                }
            });

            let itemIPv6 = new PopupMenu.PopupMenuItem(_('IP v6'));
            this.menu.addMenuItem(itemIPv4);
            this.refresh = newMenuItem(_('Refresh'));
            this.menu.addMenuItem(this.refresh);
        }
    });

export default class IndicatorExampleExtension extends Extension {
    enable() {
        this._indicator = new Indicator();
        Main.panel.addToStatusArea(this.uuid, this._indicator);
    }

    disable() {
        this._indicator.destroy();
        this._indicator = null;
    }
}
