/* extension.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
* SPDX-License-Identifier: GPL-2.0-or-later
*/

import GObject from 'gi://GObject';
import St from 'gi://St';
const { Soup, GLib } = imports.gi;

import { Extension, gettext as _ } from 'resource:///org/gnome/shell/extensions/extension.js';
import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';

import * as Main from 'resource:///org/gnome/shell/ui/main.js';

const newMenuItem = (label) => {
    return new PopupMenu.PopupMenuItem(label);
}

const Indicator = GObject.registerClass(
    class Indicator extends PanelMenu.Button {

        fetchIPs() {
            return new Promise((resolve, reject) => {
                try {
                    console.log(itemIPv4.label)
                    const message = Soup.Message.new('GET', "https://api.ipify.org");
                    const session = new Soup.Session();
                    session.send_message(message);
                    message.connect('got_chunk', (msg, chunk) => {
                        const text_decoder = new TextDecoder("utf-8");
                        const ipv4 = text_decoder.decode(chunk);
                        Main.notify(_('Whatʼs up? your ip is ' + ipv4));
                        console.log("Response: " + ipv4);
                        resolve([ipv4]);
                    });
                    message.connect('got_headers', (msg) => {
                        if (msg.status_code != Soup.Status.OK) {
                            console.log(`HTTP Status ${msg.status_code}`);
                            reject(new Error(`HTTP Status ${msg.status_code}`));
                        }
                    });
                } catch (e) {
                    console.log("Error: could not fetch public IP address: " + e);
                    reject(e);
                }
            });
        }

        _init() {
            super._init(0.0, _('MyPubIP'));
            try {
                [this.ipv4str] = this.fetchIPs();
            } catch(e) {
                console.log(e);
            }

            console.log(ipv4str);


            this.add_child(new St.Icon({
                icon_name: 'network-server-symbolic',
                style_class: 'system-status-icon',
            }));

            let itemIPv4 = newMenuItem(ipv4str);
            itemIPv4.connect('activate', async () => {
                try {
                    const message = Soup.Message.new('GET', "https://api.ipify.org");
                    const session = new Soup.Session();
                    const bytes = await session.send_and_read_async(
                        message,
                        GLib.PRIORITY_DEFAULT,
                        null,
                    );

                    if (message.get_status() != Soup.Status.OK) {
                        console.log(`HTTP Status ${message.status_code}`);
                    }

                    const text_decoder = new TextDecoder("utf-8");
                    const decoded_text = text_decoder.decode(bytes.toArray());
                    Main.notify(_('Whatʼs up? your ip is ' + decoded_text));
                    console.log("Response: " + decoded_text);

                } catch (e) {
                    console.log("Error: could not fetch public IP address: " + e);
                }
            });
            this.menu.addMenuItem(itemIPv4);

            let itemIPv6 = newMenuItem('IP v6');

            this.refresh_item = newMenuItem('Refresh');
            this.refresh_item.connect('activate', async () => {
                this.fetchIPs(this.itemIPv4, this.itemIPv6)
            });
            this.menu.addMenuItem(this.refresh_item);
        }


    });

export default class MyPubIP extends Extension {
    enable() {
        this._indicator = new Indicator();
        Main.panel.addToStatusArea(this.uuid, this._indicator);
    }

    disable() {
        this._indicator.destroy();
        this._indicator = null;
    }
}
